library ac_dart.server.app;

import 'router.dart';

import 'package:shelf/shelf.dart' hide logRequests;
import 'dart:async';
import 'package:logging/logging.dart';
import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'acdart_shelf_middleware.dart' as m;

import 'package:mojito/extend.dart' show MojitoImpl;
import 'package:mojito/mojito.dart' hide Router, router;
import 'package:atlassian_connect_shelf/src/auth/auth_impl.dart';
import 'dart:io';
import 'package:atlassian_connect_shelf/src/config/config.dart';

AcDartApp initWithConfig(ConfigFactory<AcDartConfig> configFactory,
    {EnvironmentNameResolver environmentNameResolver}) => new AcDartApp(
    configFactory, environmentNameResolver: environmentNameResolver);

AcDartApp init(String addonBaseUrlStr, AcDartHostRepoConfig repoConfig,
    {RouteCreator createRootRouter, bool logRequests: true,
    bool createRootLogger: true, IsDevMode isDevMode, bool startNgrok: false,
    String ngrokSubdomain,
    Map<Type, CustomObjectFactory> customObjects}) => new AcDartApp.simple(
    createRootRouter, logRequests, createRootLogger, isDevMode, startNgrok,
    ngrokSubdomain, addonBaseUrlStr, repoConfig, customObjects);

class AcDartApp extends MojitoImpl {
  Router get router => super.router;
  final AcDartAuthImpl auth = new AcDartAuthImpl();
  final Middleware jwtTimingLogger = m.jwtTimingLogger;

  AcDartApp._2(
      AcDartConfig config, EnvironmentNameResolver environmentNameResolver)
      : super(config, environmentNameResolver);

  AcDartApp._(ConfigFactory<AcDartConfig> configFactory,
      EnvironmentNameResolver environmentNameResolver)
      : super(MojitoImpl.resolveConfig(configFactory, environmentNameResolver),
          environmentNameResolver);

  AcDartApp(ConfigFactory<AcDartConfig> configFactory,
      {EnvironmentNameResolver environmentNameResolver})
      : this._(
          configFactory, _environmentNameResolver(environmentNameResolver));

  static EnvironmentNameResolver _environmentNameResolver(
          EnvironmentNameResolver environmentNameResolver) =>
      environmentNameResolver != null
          ? environmentNameResolver
          : environmentNameFromKey(AcDartEnvironment.AC_ENV);

  AcDartApp.simple(RouteCreator createRootRouter, bool logRequests,
      bool createRootLogger, IsDevMode isDevMode, bool startNgrok,
      String ngrokSubdomain, String addonBaseUrlStr,
      AcDartHostRepoConfig repoConfig,
      Map<Type, CustomObjectFactory> customObjects)
      : this._2(new AcDartConfig(
          server: new AcDartServerConfig(
              startNgrok: startNgrok,
              ngrokSubdomain: ngrokSubdomain,
              addonBaseUrlStr: addonBaseUrlStr,
              repoConfig: repoConfig,
              createRootRouter: createRootRouter,
              logRequests: logRequests,
              createRootLogger: createRootLogger,
              customObjects: customObjects)), isDevMode != null
          ? defaultEnvironmentNameResolver(isDevMode)
          : _environmentNameResolver(null));

  Future start({int port: 9999}) async {
    await super.start(
        port: port != null && port != 9999 ? port : config.server.serverPort);

    if (config.server.startNgrok) {
      await startNgrok(port, config.server.ngrokSubdomain);
    }

    // TODO: this is much more complicated now as we may have several addons
    // for different products
//    if (config.server.installAddonOnStartup) {
//      _log.info('installing addon...');
//      try {
//        await Future.wait(_autoRegisterHostUrls
//            .map((baseUrl) => hostService.installAddonToHost(baseUrl)));
//      } catch (e, stackTrace) {
//        _log.warning('caught error', e, stackTrace);
//      }
//    }

    return null;
  }
}

// TODO: move into config so can auto register for non prod herokus (assuming that works)
final List<Uri> _autoRegisterHostUrls = [
  Uri.parse("http://localhost:1990/confluence"),
  Uri.parse("http://localhost:2990/jira")
];

final Logger _log = new Logger('AcDartServer');

AcDartContext get acContext => context as AcDartContext;

abstract class AcDartContext implements MojitoContext<AcDartApp> {}

Future<String> startNgrok(int port, String subdomain) async {
  final Process process = await Process.start(
      'ngrok', ['--subdomain', subdomain, '-log=stdout', '$port']);
//  stdout.addStream(process.stdout);
//  stderr.addStream(process.stderr);
  process.stdout.listen((_) {});
  process.stderr.listen((_) {});
//  process.stdout.pipe(stdout);
//  process.stderr.pipe(stderr);
  print('ngrok pid: ${process.pid}');
  process.exitCode.then((code) => print('ngrok process exited with $code'));
  var hostname = 'https://$subdomain.ngrok.com';
  _log.info('sweeet - ngrok started at $hostname');
  return hostname;
}
