library ac_dart.addon_routebuilder;

import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'dart:async' show Future;
import 'package:logging/logging.dart' show Logger;
import 'package:atlassian_connect_host/host_event_model.dart';
import 'package:atlassian_connect_shelf/src/util.dart'
    show pathRelativeToAppRoot;
import 'package:constrain/constrain.dart';
import 'dart:convert';
import 'package:mojito/mojito.dart' hide Router, router;
import 'package:mojito/mojito.dart' as mojito show Router;

import 'package:atlassian_connect_shelf/src/router/hipchat/hipchat.dart';
import 'package:atlassian_connect_shelf/src/router/acdart_router.dart';
import 'package:atlassian_connect_shelf/src/router/bitbucket/bitbucket.dart';

final Logger _log = new Logger('AcDartAddonRouteBuilder');

//typedef ConnectAddon DescriptorFactory(AddonHost addonHost);
const String _DESCRIPTOR_PATH = '/atlassian-connect.json';

final Validator _validator =
    new Validator(); // TODO: this is not a shared instance!!!

final JsonEncoder _prettyJson = new JsonEncoder.withIndent('  ');

abstract class Router implements mojito.Router {
//  factory Router(Function fallbackHandler, String name, routeable,
//      RouterAdapter routerAdapter, this._addonBaseUrl,
//      this._hostRegistrationRepository, {path, middleware})
//      : super(
//      fallbackHandler, name, path, routerAdapter, routeable, middleware);
//
//  AcDartRouter.create(this._addonBaseUrl, this._hostRegistrationRepository,
//      {bool logRequests: true, Map<Type, CustomObjectFactory> customObjects})
//      : super.create();

  factory Router(Function fallbackHandler, String name, routeable,
      RouterAdapter routerAdapter, Uri addonBaseUrl,
      AcHostRegistrationRepository hostRegistrationRepository,
      {path, middleware, IsDevMode isDevMode}) = AcDartRouter;

  factory Router.create(
      Uri addonBaseUrl, AcHostRegistrationRepository hostRegistrationRepository,
      {Map<Type, CustomObjectFactory> customObjects,
      IsDevMode isDevMode}) => new AcDartRouter.create(
      addonBaseUrl, hostRegistrationRepository,
      customObjects: customObjects, isDevMode: isDevMode);

  AddonRouterBuilder get addons;
}

abstract class AddonRouterBuilder {
  HipchatRouterBuilder hipchat(String addonKey);

  /// create a child router for creating bitbucket addon routes.
  /// Pass either your own [sessionHandler] or [addonSecret] to use the default
  /// session handler
  BitbucketRouterBuilder bitbucket(
      String addonKey, ClientIdFactory oauthClientIdFactory,
      {path: 'bitbucket', SessionHandler sessionHandler, String addonSecret});
}

class AcDartRequestContext {
  final AcHostService hostService;

  AcDartRequestContext(this.hostService);

  AcPrincipal get principal => authenticatedContext()
      .map((AuthenticatedContext c) => c.principal)
      .getOrElse(() => null);

  Future<AcHostHttpClient> get httpClient =>
      hostService.httpClientForHostKey(principal.hostKey);

  Future<AcHostRegistration> get hostRegistration async =>
      (await hostService.findRegistrationByHostKey(principal.hostKey))
          .getOrElse(() => null);

  Future<ClientId> get clientId async => (await hostRegistration).clientId;
}

final String _assetDir = pathRelativeToAppRoot('asset');

/// Creates an AcDart router
Router router(Uri addonBaseUrl,
        AcHostRegistrationRepository hostRegistrationRepository,
        {Map<Type, CustomObjectFactory> customObjects, IsDevMode isDevMode}) =>
    new Router.create(addonBaseUrl, hostRegistrationRepository,
        customObjects: customObjects, isDevMode: isDevMode);
