// Copyright (c) 2014, The Mojito project authors.
// Please see the AUTHORS file for details.
// All rights reserved. Use of this source code is governed by
// a BSD 2-Clause License that can be found in the LICENSE file.

library acdart.auth;

import 'package:shelf_auth/shelf_auth.dart';
import 'package:mojito/mojito.dart';
import 'package:atlassian_connect_shelf/src/auth.dart';
import 'dart:async';
import 'package:option/option.dart';
import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'package:logging/logging.dart';

Logger _log = new Logger('acdart.auth');

abstract class AcDartAuth extends MojitoAuth {
//  /// builder for authentication middleware to be applied all routes
//  AcDartAuthenticationBuilder get global;

  /// builder for authentication middleware that you choose where to include
  AcDartAuthenticationBuilder builder();
}

class AcDartAuthenticationBuilder extends AuthenticationBuilder {
  AcDartAuthenticationBuilder();

  AcDartAuthenticationBuilder productHostAuthenticator() =>
      authenticator(new JwtAuthenticator());

  AcDartAuthenticationBuilder sessionTokenAuthenticator() =>
      authenticator(new SessionTokenAuthenticator());

  // TODO: should expose these params
  AcDartAuthenticationBuilder addonSession(
//          {Duration idleTimeout: const Duration(minutes: 30),
//          Duration totalSessionTimeout: const Duration(days: 1)}
      ) => sessionHandler(new SessionTokenSessionHandler());
}

class NotAcDartUnknownSubjectAuthoriser implements Authoriser {
  Future<bool> isAuthorised(Request request) =>
      new Future.value(_isAuthorised(request));

  bool _isAuthorised(Request request) {
    final authContext = getAuthenticatedContext(request);
    final isNotAcDartUnknown =  authContext is None ||
        authContext.get().principal.name != acDartUnknownSubject;

    if (!isNotAcDartUnknown) {
      _log.fine('denying access to AcDart "unknown" user');
    }

    return isNotAcDartUnknown;
  }
}
