library ac_dart.configuration;

import 'package:atlassian_connect_host/src/preconditions.dart';
import 'package:logging/logging.dart';
import 'package:atlassian_connect_host/host_registration_persistence.dart';
import 'package:atlassian_connect_host/src/host_registration_mongo_persistence.dart';
import 'dart:io';
import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'package:mojito/mojito.dart' hide Router, router;
import 'package:atlassian_connect_shelf/router.dart';
import 'package:quiver/core.dart';
import 'package:constrain/constrain.dart';
import 'package:atlassian_connect_shelf/src/util.dart';
import 'package:appengine/appengine.dart' as appengine;

final List<Uri> _autoRegisterHostUrls = [
  Uri.parse("http://localhost:1990/confluence"),
  Uri.parse("http://localhost:2990/jira")
];

final Logger _log = new Logger('AcDartConfig');

/// The set of environment properties for AcDart
class AcDartEnvironment {
  static const String AC_ENV = 'AC_ENV';
  static const String AC_ADDON_KEY = 'AC_ADDON_KEY';
//  static const String AC_ADDON_SECRET = 'AC_ADDON_SECRET';
  static const String AC_SERVER_PORT = 'PORT'; // TODO: move to Mojito
  static const String AC_BASE_URL = 'AC_BASE_URL';
  static const String AC_MONGO_HOST_URL = 'AC_MONGO_HOST_URL';
  static const String AC_ROOT_LEVEL = 'AC_ROOT_LEVEL'; // TODO: move to Mojito
  static const String AC_LOG_LEVEL = 'AC_LOG_LEVEL';

  static final String addonKey = _fromEnvironment(AC_ADDON_KEY);

//  static final String addonSecret = _fromEnvironment(AC_ADDON_SECRET);

  static final String baseUrl = _fromEnvironment(AC_BASE_URL);

  static final String mongoUrlStr = _fromEnvironment(AC_MONGO_HOST_URL);

  static String _fromEnvironment(String key) {
    final String systemProperty = new String.fromEnvironment(key);
    if (systemProperty != null) {
      return systemProperty;
    }

    return Platform.environment[key];
  }
}

/// The environments that AcDart knows about
class EnvironmentNames {
  static const String development = 'development';
  static const String production = 'production';
}

class _RouteCreatorCludge {
  AcDartServerConfig config;

  Router call() =>
      new Router.create(config.addonBaseUrl, config.repoConfig.hostRepo,
          customObjects: config.customObjects);
}

@Ensure(_ngrokSubdomainMandatoryIfUsingNgrok,
    description: 'must provide a ngrokSubdomain if startNgrok is true')
class AcDartServerConfig extends MojitoServerConfig {
  @NotNull()
  final bool installAddonOnStartup;

  @NotNull()
  final bool startNgrok;

  final String ngrokSubdomain;

//  final int serverPort; in mojito
//  final ClientId oauthClientId;

//  // TODO: this is really a server secret or us secret. Could be supported
//  // in mojito as it is related to using jwt sessions
//  final String addonSecret;

  final String addonBaseUrlStr;

  final Map<Type, CustomObjectFactory> customObjects;

  @NotNull()
  Uri get addonBaseUrl {
    final _addonBaseUrlStr = addonBaseUrlStr != null
        ? addonBaseUrlStr
        : startNgrok != null && startNgrok ? ngrokUrlStr(ngrokSubdomain) : null;

//    print('----- $addonBaseUrlStr');
    return _addonBaseUrlStr != null ? Uri.parse(_addonBaseUrlStr) : null;
  }

  @NotNull()
  final AcDartHostRepoConfig repoConfig;

  AcDartServerConfig(
      {RouteCreator createRootRouter,
      bool logRequests: true,
      bool createRootLogger: true,
      int serverPort: 9999,
      this.installAddonOnStartup: false,
      this.startNgrok: false,
      this.ngrokSubdomain,
      this.repoConfig,
      String addonBaseUrlStr,
      this.customObjects})
      : this.addonBaseUrlStr =
            firstNonNullOrNull(addonBaseUrlStr, AcDartEnvironment.baseUrl),
        super(
            createRootRouter: createRootRouter != null
                ? createRootRouter
                : new _RouteCreatorCludge(),
            createRootLogger: createRootLogger,
            logRequests: logRequests,
            serverPort: serverPort) {
    // TODO: damn this is ugly, but a chicken and egg problem
    if (this.createRootRouter is _RouteCreatorCludge) {
      (this.createRootRouter as _RouteCreatorCludge).config = this;
    }
  }

  AcDartServerConfig.development(
      {bool installAddonOnStartup: true,
      bool startNgrok: true,
      String ngrokSubdomain,
      AcDartHostRepoConfig repoConfig})
      : this(
            repoConfig:
                firstNonNull(repoConfig, new AcDartHostRepoConfig.inMemory()),
            installAddonOnStartup: installAddonOnStartup,
            startNgrok: startNgrok,
            ngrokSubdomain: ngrokSubdomain);

  @override
  AcDartServerConfig merge(AcDartServerConfig other) {
    return new AcDartServerConfig(
        createRootRouter:
            firstNonNull(other.createRootRouter, createRootRouter),
        logRequests: firstNonNull(other.logRequests, logRequests, true),
        createRootLogger:
            firstNonNull(other.createRootLogger, createRootLogger, true),
        serverPort: firstNonNull(other.serverPort, serverPort, 9999),
        installAddonOnStartup: firstNonNull(
            other.installAddonOnStartup, installAddonOnStartup, false),
        addonBaseUrlStr: firstNonNull(
            other.addonBaseUrlStr, addonBaseUrlStr, AcDartEnvironment.baseUrl),
        startNgrok: firstNonNull(other.startNgrok, startNgrok, false),
        ngrokSubdomain: firstNonNull(other.ngrokSubdomain, ngrokSubdomain),
        repoConfig: firstNonNull(other.repoConfig, repoConfig),
        customObjects:
            firstNonNull(other.customObjects, customObjects, const {}));
  }
}

bool _ngrokSubdomainMandatoryIfUsingNgrok(AcDartServerConfig config) =>
    config.ngrokSubdomain != null || !config.startNgrok;

//bool _baseUrlIsMandatoryUnlessUsingNgrok(AcDartConfig config) =>
//    config.addonBaseUrl != null || config.addonServer.startNgrok;

/**
 *  A configuration set for an AcDart addon instance.
 *
 *  The [development] constructor provides defaults suitable for a development
 *  environment. Each of these can be overridden if desired but the aim is to
 *  provide as much defaulting as possible to get started quickly.
 *
 *  The main constructor is for use for the other configuration sets such as
 *  the common values and production. It provides much less defaulting.
 */
//@Ensure(_baseUrlIsMandatoryUnlessUsingNgrok,
//    description: 'must provide a base url, set AC_BASE_URL environment '
//    'variable or system property, or set startNgrok to true')
class AcDartConfig<A extends Config> extends MojitoConfig {
  AcDartConfig({AcDartServerConfig server, A app})
      : super(server: server, app: app);

  AcDartConfig.development({AcDartServerConfig server, A app})
      : this(
            server: firstNonNull(server, new AcDartServerConfig.development()),
            app: app);

  /**
   * Anything in [other] takes precedence.
   * If no value exists in either then many fields will fallback to an
   * environment variable
   */
  AcDartConfig<A> merge(AcDartConfig<A> other) {
    final AcDartServerConfig mergedServer =
        mergeChildConfigs(server, other.server);

    final newServer = mergedServer.createRootRouter != null
        ? mergedServer
        : mergedServer.change(
            createRootRouter: () => new Router.create(
                mergedServer.addonBaseUrl, mergedServer.repoConfig.hostRepo,
                customObjects: mergedServer.customObjects));

    return new AcDartConfig<A>(
        server: newServer, app: mergeChildConfigs(app, other.app));
  }
}

/// Configuration for [AcHostRegistrationRepository] instances. Provides
/// convenience constructors for setting up the out of the box implementations
/// (in memory, local file and mongo)
class AcDartHostRepoConfig {
  static final String defaultTestMongoUrlStr = "mongodb://127.0.0.1/acDart";
  final AcHostRegistrationRepositoryFactory _hostRepoFactory;
  AcHostRegistrationRepository get hostRepo => _hostRepoFactory();

  AcDartHostRepoConfig(AcHostRegistrationRepositoryFactory hostRepoFactory)
      : this._hostRepoFactory = checkNotNull(hostRepoFactory, 'hostRepo');

  /// Suitable for dev only
  AcDartHostRepoConfig.inMemory()
      : this(() => new AcHostRegistrationInMemoryTestRepository());

  /// Suitable for dev only
  AcDartHostRepoConfig.localFile(File file)
      : this(() => new AcHostRegistrationLocalFileTestRepository(file));

  AcDartHostRepoConfig.mongo({String dbUrl, Db db})
      : this(() => new AcHostRegistrationMongoRepository(
            db != null ? db : new Db(_dbUrl(dbUrl))));

  AcDartHostRepoConfig.datastore({DatastoreDBProvider dbProvider})
      : this(() => new AcHostRegistrationDatastoreRepository(
            dbProvider ?? () => appengine.context.services.db));

  static String _dbUrl(String dbUrl) {
    return dbUrl != null ? dbUrl : AcDartEnvironment.mongoUrlStr;
  }
}

String ngrokUrlStr(String subdomain) => 'https://${subdomain}.ngrok.com';
