library ac_dart.descriptor.builder.hipchat;

import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'package:mojito/mojito.dart'
    show ClientId, DefaultRoutePath, DefaultRouter, Oauth2RouteNames;
import 'package:uri/uri.dart';
import 'package:option/option.dart';
import 'package:atlassian_connect_shelf/src/descriptor_builders/descriptor_builder.dart';

class HipchatDescriptorBuilder
    implements
        DescriptorBuilder<Capabilities, HipchatScope, HipchatConnectAddon> {
  final List<_WebhookSpec> _webhooks = [];
  Oauth2RouteNames _routeNames;
  final String addonKey;
  final Uri baseUrl;

  HipchatDescriptorBuilder(this.addonKey, this.baseUrl);

  void addWebhook(String format, String name, RegExp pattern) {
    _webhooks.add(new _WebhookSpec(format, name, pattern));
  }

  void setOAuthRouteNames(Oauth2RouteNames routeNames) {
    _routeNames = routeNames;
  }

  HipchatConnectAddon build(DefaultRouter rootRouter, Vendor vendor,
      {String name,
      String description,
      ClientId oauthClientId,
      Iterable<HipchatScope> scopes}) {
    final webhooks = _webhooks.map((spec) => createWebhook(spec, rootRouter));

    return new HipchatConnectAddon(
        key: addonKey,
        name: 'so interesting',
        description: 'like wow man',
        capabilities: new Capabilities(
            hipchatApiConsumer:
                new HipchatApiConsumer(["send_notification", "admin_room"]),
            installable: _buildInstallable(rootRouter),
            oauth2Provider: _buildOauth2Provider(rootRouter),
            webhooks: webhooks));
  }

  Installable _buildInstallable(DefaultRouter rootRouter) {
    return _getAbsoluteRouteUrl(
            rootRouter, DescriptorBuilder.installationRouteName)
        .map((Uri callbackUrl) => new Installable(
            allowGlobal: true, allowRoom: true, callbackUri: callbackUrl))
        .getOrElse(() => null);
  }

  Option<Uri> _getAbsoluteRouteUrl(DefaultRouter rootRouter, String routeName) {
    return rootRouter.findByName(routeName).map((DefaultRoutePath rp) {
      final Uri path = rp.fullPath.expand({});
      return (new UriBuilder.fromUri(baseUrl)..path += path.toString()).build();
    });
  }

  Uri _getMandatoryAbsoluteRouteUrl(
      DefaultRouter rootRouter, String routeName) {
    return _getAbsoluteRouteUrl(rootRouter, routeName)
        .getOrElse(() => throw new StateError('missing route $routeName'));
  }

  Oauth2Provider _buildOauth2Provider(DefaultRouter rootRouter) {
    if (_routeNames == null) {
      return null;
    }

    final userGrantRoute =
        _getMandatoryAbsoluteRouteUrl(rootRouter, _routeNames.userGrantRoute);
    final authTokenRoute =
        _getMandatoryAbsoluteRouteUrl(rootRouter, _routeNames.authTokenRoute);

    return new Oauth2Provider(
        authorizationUrl: userGrantRoute, tokenUrl: authTokenRoute);
  }
}

HipchatWebhook createWebhook(_WebhookSpec spec, DefaultRouter rootRouter) {
  final Uri url = rootRouter
      .findByName(spec.name)
      .map((DefaultRoutePath rp) => rp.fullPath.expand({}))
      .getOrElse(() => throw new StateError(
          "Doh, can't find webhook route with name ${spec.name}"));

  return spec.toWebhook(url);
}

class _WebhookSpec {
  final String event, name;
  final RegExp pattern;

  _WebhookSpec(this.event, this.name, this.pattern);

  HipchatWebhook toWebhook(Uri url) =>
      new HipchatWebhook(event: event, name: name, pattern: pattern, url: url);
}
