library ac_dart.addon_routebuilder.impl;

import 'package:shelf/shelf.dart';
import 'package:atlassian_connect_host/atlassian_connect_host.dart';
import 'package:logging/logging.dart';
import 'package:mojito/mojito.dart' hide Router;
import 'package:mojito/src/router_impl.dart';
import 'package:atlassian_connect_shelf/router.dart';
import 'package:atlassian_connect_shelf/src/router/hipchat/hipchat.dart';
import 'package:atlassian_connect_shelf/src/router/hipchat/hipchat_impl.dart';
import 'package:atlassian_connect_shelf/src/router/bitbucket/bitbucket.dart';
import 'package:atlassian_connect_shelf/src/router/bitbucket/bitbucket_impl.dart';
import 'package:quiver/check.dart';

final Logger _log = new Logger('AcDartAddonRouteBuilder');

class AcDartRouter extends MojitoRouterBuilder implements Router {
  final AcHostRegistrationRepository _hostRegistrationRepository;
  final Uri _addonBaseUrl;

  AcDartRouter(
      Function fallbackHandler,
      String name,
      routeable,
      RouterAdapter routerAdapter,
      this._addonBaseUrl,
      this._hostRegistrationRepository,
      {path,
      middleware,
      IsDevMode isDevMode})
      : super(fallbackHandler, name, path, routerAdapter, routeable, middleware,
            isDevMode);

  AcDartRouter.create(this._addonBaseUrl, this._hostRegistrationRepository,
      {Map<Type, CustomObjectFactory> customObjects, IsDevMode isDevMode})
      : super.create(
            handlerAdapter: customObjects != null && customObjects.isNotEmpty
                ? handlerAdapter(customObjects: customObjects)
                : null,
            isDevMode: isDevMode);

  @override
  AcDartRouter createChild(String name, path, routeable,
      RouterAdapter routerAdapter, Middleware middleware) {
    return new AcDartRouter(fallbackHandler, name, routeable, routerAdapter,
        _addonBaseUrl, _hostRegistrationRepository,
        path: path, middleware: middleware, isDevMode: isDevMode);
  }

  @override
  AddonRouterBuilder get addons => new AddonRouterBuilderImpl(
      this, _addonBaseUrl, _hostRegistrationRepository, isDevMode);
}

class AddonRouterBuilderImpl implements AddonRouterBuilder {
  final Router router;
  final AcHostRegistrationRepository _hostRegistrationRepository;
  final Uri _addonBaseUrl;
  final IsDevMode _isDevMode;

  AddonRouterBuilderImpl(this.router, this._addonBaseUrl,
      this._hostRegistrationRepository, this._isDevMode);

  HipchatRouterBuilder hipchat(String addonKey,
          {path: 'hipchat',
          SessionHandler sessionHandler,
          String addonSecret}) =>
      new HipchatRouterBuilderImpl(router, path, addonKey, _addonBaseUrl,
          _hostRegistrationRepository, _isDevMode,
          sessionHandler: sessionHandler, addonSecret: addonSecret);

  BitbucketRouterBuilder bitbucket(
          String addonKey, ClientIdFactory oauthClientIdFactory,
          {path: 'bitbucket',
          SessionHandler sessionHandler,
          String addonSecret}) =>
      new BitbucketRouterBuilderImpl(
          router,
          path,
          addonKey,
          _addonBaseUrl,
          _hostRegistrationRepository,
          checkNotNull(oauthClientIdFactory),
          _isDevMode,
          sessionHandler: sessionHandler,
          addonSecret: addonSecret);
}
